@@ -14,27 +14,27 @@ my @points;
 sub lines { 1 }
 sub cols  { 1 }
 
-sub render_to_rb
+use constant CLEAR_BEFORE_RENDER => 0;
+sub render
 {
    my $self = shift;
-   my ( $rb, $rect ) = @_;
-
    my $win = $self->window;
 
-   $rb->eraserect( $rect );
+   $win->clear;
    foreach my $point ( @points ) {
-      $rb->text_at( $point->[0], $point->[1], "X" );
+      $win->goto( $point->[0], $point->[1] );
+      $win->print( "X" );
    }
 }
 
 sub on_mouse
 {
    my $self = shift;
-   my ( $args ) = @_;
+   my ( $ev, $button, $line, $col ) = @_;
 
-   return unless $args->type eq "press" and $args->button == 1;
+   return unless $ev eq "press" and $button == 1;
 
-   push @points, [ $args->line, $args->col ];
+   push @points, [ $line, $col ];
    shift @points while @points > 10;
    $self->redraw;
 }
